package api

enum class ApiError {
    UNKNOWN_ERROR,
    BAD_PARAMETER,
    INTERNAL_ERROR,
    INVALID_DELETE_DATE,
    DATA_TOO_BIG,
    NOT_FOUND;

    fun raise(text: String? = null): Nothing {
        throw ApiException(this, text ?: defaultText())
    }

    fun defaultText(): String = name.lowercase().replace('_', ' ')
}

class ApiException(val code: ApiError, _text: String? = null, cause: Throwable? = null) :
    Exception(_text ?: code.defaultText(), cause) {
    val text by lazy { _text ?: code.defaultText() }
}


@kotlinx.serialization.Serializable
data class ErrorResult(val code: ApiError,val text: String) {
    fun raise(): Nothing { throw ApiException(code, text) }
}