package tools

import api.ApiError
import api.ApiException

class CatchApiResult<T>(_result: T?, val exception: ApiException?) {
    var errorIsProcessed = false
        private set

    var result = _result
        private set

    suspend fun on(vararg errors: ApiError, block: suspend () -> T?): CatchApiResult<T> {
        if (exception != null && exception.code in errors) {
            errorIsProcessed = true
            result = block()
        }
        return this
    }

    fun ignore(vararg errors: ApiError): CatchApiResult<T> {
        if (exception != null && exception.code in errors)
            errorIsProcessed = true
        return this
    }

    fun orThrowError(): CatchApiResult<T> {
        if (exception != null && !errorIsProcessed) throw exception
        return this
    }

    fun get(): T? = orThrowError().result
}


suspend fun <T> catchApi(block: suspend () -> T): CatchApiResult<T> =
    try {
        CatchApiResult(block(), null)
    } catch (e: ApiException) {
        CatchApiResult(null, e)
    } catch (t: Throwable) {
        CatchApiResult(null, ApiException(ApiError.UNKNOWN_ERROR, cause= t))
    }